; The recipe for this calculation is laid out in some detail
; in Hanasoge 2008, ApJ, 680, 1457
;
; The code requires a large number of user inputs - going through
; the various comments may prove instructive. 
;
; N: number of points in the background solar model (in z)
N = 300


; Background stratification (must be convectively stable!)
; The data must be in an ascii file, and arranged according to:
; r/R, c, rho, p, gravity, gamma_1   (all in CGS units)
backmodel = 'model_S'
;backmodel = 'solar_model'

; REPROJECTION (FROM CYLINDRICAL TO CARTESIAN)
; Eventual output parameters:
; If you want to reproject, set reproject =1 
; 
 reproject = 1

; The reprojection from cylindrical to Cartesian coordinates
; will be in a box of (Nx, Nx, N) grid points and length L (in Mm).
Nx= 100
L = 145.77

; Reprojection output:
directory = ''

data=DBLARR(6,N)
; Need to read in the background solar model over here:
openr,1,backmodel
readf,1,data
close,1

; Defining constants and fundamental variables
Rsol=6.959894677E+10
r = data[0,*]
rho0 = data[2,*]
p0 = data[3,*]
c_speed = data[1,*]
surf = WHERE(ABS(r- 1.0) EQ MIN(ABS(r-1.0)) )
dimrho = rho0[surf[0]]
dimc = c_speed[surf[0]]
dimp = dimrho * dimc^2.
rho0 = rho0/dimrho
p0 = p0/(dimrho * dimc^2.)
grav = data[4,*]*Rsol/dimc^2.
gam = data[5,*]
; All variables are non-dimensionalized

; IN CASE YOU MERELY WANT TO REPROJECT, AND DO NOT
; WANT TO COMPUTE THE SCHLUTER-TEMESVARY RESULT, SET
; schluter = 0

SCHLUTER = 1
IF (SCHLUTER EQ 1) THEN BEGIN

; B0 is \psi(z) in the paper ( Hanasoge 2008)
; r is the vertical coordinate: r =1.0 at the photosphere - normalized by R_\odot
; rad - is the horizontal coordinate, also normalized by R_\odot
;B0 = exp((1. - r)/(20./696.))*(alog(2)*69.6)^2. ; 3000 G version

B0 = exp((1. - r)/(8./696.))*(alog(2)*696/6.)^2. ; 3000 G version

; This is M from the paper - determines the magnitude of the field at the photosphere.
; Change 3000 (in Gauss) to whatever value you want.

 const =  800./(dimp^0.5 * B0[surf[0]])

;
; EXAMPLES
;
; To recover the thin flux tube:
; B0 = 1./(2. *Radii^2.) ; use parameter values of maxrad = 0.408, minrad = 0.25 and the formula for Radii above
; Radii[*] = (minrad + (maxrad-minrad)/(1. + exp((1.0015 - r)/0.0035)))/696. ; Formula for thin flux tube
; reduction = 1.0/(1.0 + 0.05*(bz[0,*]^2.)/(dimp*p0[*]) ) ; reduction for the 1000 G thin flux tube
; const =  1000./(dimp^0.5 * B0[surf[0]])
; The flux of the thin flux tube is 0.61 kG Mm^2, almost precisely the same as the measured value
; in Duvall et al. (2006)
;

; To recover the thick 3000G flux tube:
; B0 = exp((1. - r)/(10./696.))*(alog(2)*69.6)^2. ; 3000 G version
; reduction = 1.0/(1.0 + 0.034*(bz[0,*]^2.)/(dimp*p0[*]) ) ; reduction for the 3000 G thick flux tube
; const = 3000./(dimp^0.5 * B0[surf[0]])
;

; BEGIN CALCULATIONS
;

Phi = !dPI*const
db0dz = deriv(r,b0)
d2b0dz2 = deriv(r,db0dz)
term1_eq1 = -0.25*B0^(-1.)*db0dz^2. + 0.5*d2b0dz2

; number of points in the horizontal direction
Nrad = 400
;Nrad = 500
reduction = DBLARR(N)
;reduction[*] = 1.0/(1.0 + exp(-(1.00025 - r)/0.0001))
Kernel = DBLARR(Nrad,N)
; below is the horizontal grid in cyl coords, normalized by solar rad. e.g. much finer grid spacing 
; required for thin flux tubes (~0.01-0.03) depending on the typ of thin flux tube you require
rad = DINDGEN(Nrad)*0.3/695.9894677
EX = DBLARR(Nrad,N)

FOR I=0,N-1 DO BEGIN
EX[*,I] = EXP(-rad^2.*B0[I]) * const
KERNEL[*,I] = ( 0.5*(rad^3*(db0dz[I])^2.)*EX[*,I] - 0.5*(rad*d2b0dz2[I])*EX[*,I] + 2.0*(rad*B0[I]^2.)*EX[*,I] ) 
ENDFOR

Integrand = KERNEL* EX

bz = DBLARR(Nrad,N)
br = DBLARR(Nrad,N)

; according to the formulae of Hanasoge 2008

FOR I=0,N-1 DO BEGIN
 bz[*,I] = EX[*,I] * B0[I] * dimp^0.5 ;* const
ENDFOR

FOR I=0, Nrad -1 DO BEGIN
 br[I,*] = - 0.5*rad[I] * db0dz * EX[I,*] * dimp^0.5 ;* const
ENDFOR

; Reduction is \zeta(z) from the paper
; I use the prescription of Robert Cameron - Cameron et al. (2007) 
; Note that the constant preceding the bz[0,*]^2. term determines how
; quickly the Lorentz force is damped. Increase the constant for greater
; damping and vice-versa. A rule of thumb is to leave this term as small
; as possible to ensure that the solutions aren't too corrupted.
; Check min(rho)~1e-16 and min(p)~1e-5 also check that no (-)ve p or rho appears after reprojection 
;reduction = 1.0/(1.0 + 0.038*(bz[0,*]^2.)/(dimp*p0[*]) ) 

;reduction = 1.0/(1.0 + 0.065*(bz[0,*]^2.)/(dimp*p0[*]) ) 

;reduction = 1.0/(1.0 + 0.035*(bz[0,*]^2.)/(dimp*p0[*]) ) ; reduction for the 3000 G thick flux tube
reduction = 1./(1. + 0.000005*exp((dindgen(300) - 270.)))


; Continue calculations

pi = p0 + const^2.*reduction/(4.*!dpi)*(- B0^2.*0.5 - B0^(-1.)*db0dz^2.*0.0625 + d2b0dz2*0.125) ;const^2./(8.*!dPI) *( 0.5*term1_eq1 - B0^2.) + p0

p = DBLARR(Nrad, N)
p[0,*] = pi
FOR K=0,N-1 DO BEGIN
 FOR I=1,Nrad-1 DO BEGIN
  p[I,K] = pi[K] + B0[K]*reduction[K]/(4.*!dPI) * INT_TABULATED(rad[0:I], INTEGRAND[0:I,K], /DOUBLE)
 ENDFOR
ENDFOR 

rho = DBLARR(Nrad, N)
FOR I=0,Nrad-1 DO BEGIN
 rho[I,*]  = 1.0/grav*( -DERIV(r,p[I,*]) + reduction/(8.*!dPI) * rad[I] * EX[I,*] * KERNEL[I,*] * db0dz)
ENDFOR
print,'The flux in kG.Mm.Mm is :'
print,2.*!dpi*(total(bz[1:298,236]*rad[1:298]) + 0.5*(bz[0,236]*rad[0] + bz[299,236]*rad[299]))*(695.9894)^2.*0.001 * (rad[1] - rad[0])

p = p*(dimrho * dimc^2.)
rho = rho*dimrho
grav = grav*dimc^2./Rsol
p0 = p0*dimp
rho0 = rho0*dimrho

print,min(p),min(rho)
;read,pause
IF ((MIN(p) LE 10.^(-6.)) OR (MIN(rho) LE 10.^(-15.))) THEN BEGIN
print,'The pressure or density is (are) either negative or too close to zero for comfort'
print,'Stopping code'
;read,pause
ENDIF
print,'Writing out the reduction file: filename is "reduction"'


filename = 'reduction'
OPENW,1,filename
FOR i=0,N-1 DO BEGIN
PRINTF,1,reduction[i],format='(1E21.14)'
ENDFOR
CLOSE,1
print, 'Schluter-Temesvary approximation successfully computed'
nrho=rho
np=p

; For Hamed
;for j=0,nrad-1 do begin
;p[j,*]=smooth(p[j,*],20)
;rho[j,*]=smooth(rho[j,*],20)
;endfor
;read,pause
;rv = (r-1.)*Rsol*10.^(-8.)
;rh = rad*Rsol*10.^(-8.)
;OPENW,1,'model3000g_Nov_smooth20'
;FOR i=0,N-1 DO BEGIN
; FOR J=0,Nrad-1 DO BEGIN
;PRINTF,1,rh[J],rv[I],rho[J,I], p[J,I],br[J,I],bz[J,I], grav[I],gam[I], reduction[i],nrho[J,I],np[J,I],format='(11E21.14)'
; ENDFOR
;ENDFOR
;CLOSE,1


ENDIF

; Note, if you haven't computed the Schluter-Temesary approximation,
; and just want to reproject, please read in the pressure, density,
; magnetic field and the corresponding grid below.


IF (REPROJECT EQ 1) THEN BEGIN

;
; Now reprojecting the magnetic field onto an appropriate x-y-z grid
; ny =nx
; L is the horizontal side in Mm
; directory is the output file directory
temp = DBLARR(Nx,Nx,N)
x = DINDGEN(Nx)/(Nx-1.) *L - L *0.5
rad = rad*695.9894677
Nmax_rad = WHERE(ABS(rad[Nrad-1]- x) EQ MIN(ABS(rad[Nrad-1]-x)) ) - 1

Nmax_rad_neg = WHERE(ABS(rad[Nrad-1] + x) EQ MIN(ABS(rad[Nrad-1] + x)) ) + 1


rdi = DBLARR(Nmax_rad-Nx/2+1,Nmax_rad-Nx/2+1)
cosang = DBLARR(Nmax_rad-Nx/2+1,Nmax_rad-Nx/2+1)
sinang = DBLARR(Nmax_rad-Nx/2+1,Nmax_rad-Nx/2+1)

;lag = DBLARR(N)
;FOR I=0,N-1 DO BEGIN
; lag[I] = MEAN(rho[200:*,I]) - rho0[I]
;ENDFOR

FOR K=0,N-1 DO BEGIN
 temp[*,*,K]=p0[K] 
ENDFOR

FOR J=Nx/2,Nmax_rad[0] DO BEGIN

 rdi[*,J-Nx/2] = (x[Nx/2:Nmax_rad[0]]^2. + x[J]^2.)^0.5
 cosang[*,J-Nx/2] = x[Nx/2-1:Nmax_rad[0]-1]/rdi[*,J-Nx/2]
 sinang[*,J-Nx/2] = x[J]/rdi[*,J-Nx/2]

 FOR I=Nx/2, Nmax_rad[0] DO BEGIN
  FOR K=0,N-1 DO BEGIN
   temp[I,J,K] = INTERPOL(p[*,K],rad,rdi[I-Nx/2,J-Nx/2]); -lag[K];use linear interpolation. No need to use LSQUADRATIC. These are all smooth functions
  ENDFOR
 ENDFOR

ENDFOR

temp[Nx/2:*,0:Nx/2-1,*] = REVERSE(temp[Nx/2:*,Nx/2:*,*],2)
temp[0:Nx/2-1,Nx/2:*,*] = REVERSE(temp[Nx/2:*,Nx/2:*,*],1)
temp[0:Nx/2-1,0:Nx/2-1,*] = REVERSE(temp[Nx/2:*,0:Nx/2-1,*],1)


writefits,directory+'pressure.fits',temp
print,min(temp)

lag = DBLARR(N)
FOR I=0,N-1 DO BEGIN
 lag[I] = MEAN(rho[200:*,I]) - rho0[I]
ENDFOR

FOR K=0,N-1 DO BEGIN
 temp[*,*,K]=rho0[K] 
ENDFOR

FOR J=Nx/2,Nmax_rad[0] DO BEGIN
 rdi[*,J-Nx/2] = (x[Nx/2:Nmax_rad]^2. + x[J]^2.)^0.5
 FOR I=Nx/2, Nmax_rad[0] DO BEGIN
  FOR K=0,N-1 DO BEGIN
   temp[I,J,K] = INTERPOL(rho[*,K],rad,rdi[I-Nx/2,J-Nx/2])-lag[K];use linear interpolation. No need to use LSQUADRATIC. These are all smooth functions
  ENDFOR
 ENDFOR
ENDFOR

temp[Nx/2:*,0:Nx/2-1,*] = REVERSE(temp[Nx/2:*,Nx/2:*,*],2)
temp[0:Nx/2-1,Nx/2:*,*] = REVERSE(temp[Nx/2:*,Nx/2:*,*],1)
temp[0:Nx/2-1,0:Nx/2-1,*] = REVERSE(temp[Nx/2:*,0:Nx/2-1,*],1)

writefits,directory+'density.fits',temp
print,min(temp)

temp[*,*,*] = 0.0

FOR J=Nx/2,Nmax_rad[0] DO BEGIN
 rdi[*,J-Nx/2] = (x[Nx/2:Nmax_rad]^2. + x[J]^2.)^0.5
 FOR I=Nx/2, Nmax_rad[0] DO BEGIN
  FOR K=0,N-1 DO BEGIN
   temp[I,J,K] = INTERPOL(bz[*,K],rad,rdi[I-Nx/2,J-Nx/2]);use linear interpolation. No need to use LSQUADRATIC. These are all smooth functions
  ENDFOR
 ENDFOR
ENDFOR

temp[Nx/2:*,0:Nx/2-1,*] = REVERSE(temp[Nx/2:*,Nx/2:*,*],2)
temp[0:Nx/2-1,Nx/2:*,*] = REVERSE(temp[Nx/2:*,Nx/2:*,*],1)
temp[0:Nx/2-1,0:Nx/2-1,*] = REVERSE(temp[Nx/2:*,0:Nx/2-1,*],1)
;read,pause
writefits,directory+'B_z.fits',temp

temp[*,*,*] = 0.0
tempy = DBLARR(Nx,Nx,N)

FOR J=Nx/2,Nmax_rad[0] DO BEGIN
 rdi[*,J-Nx/2] = (x[Nx/2:Nmax_rad]^2. + x[J]^2.)^0.5
 FOR I=Nx/2, Nmax_rad[0] DO BEGIN
  FOR K=0,N-1 DO BEGIN
;   temp[I,J,K] = INTERPOL(br[*,K],rad,rdi[I-Nx/2,J-Nx/2])*cosang[I-Nx/2,J-Nx/2]
   tempy[I,J,K] = INTERPOL(br[*,K],rad,rdi[I-Nx/2,J-Nx/2])*sinang[I-Nx/2,J-Nx/2]
;   temp[J,I,K] = tempy[I,J,K]
  ENDFOR
 ENDFOR
ENDFOR

;temp[0:Nx/2-1,Nx/2:Nx-1,*] = -REVERSE(temp[Nx/2:*,Nx/2:*,*],1)
;temp[0:Nx/2-1,0:Nx/2-1,*] = REVERSE(temp[0:Nx/2-1,Nx/2:Nx-1,*],2)
;temp[Nx/2:Nx-1,0:Nx/2-1,*] = REVERSE(temp[Nx/2:Nx-1,Nx/2:Nx-1,*],2)

tempy[0:Nx/2-1,Nx/2:Nx-1,*] = REVERSE(tempy[Nx/2:*,Nx/2:*,*],1)
tempy[0:Nx/2-1,0:Nx/2-1,*] = -REVERSE(tempy[0:Nx/2-1,Nx/2:Nx-1,*],2)
tempy[Nx/2:Nx-1,0:Nx/2-1,*] =-REVERSE(tempy[Nx/2:Nx-1,Nx/2:Nx-1,*],2)

FOR J=0,Nx-1 DO BEGIN
 FOR I=0, Nx-1 DO BEGIN
  temp[J,I,*] = tempy[I,J,*]
 ENDFOR
ENDFOR

;tempy[Nx/2:*,0:Nx/2-1,*] = REVERSE(tempy[Nx/2:*,Nx/2:*,*],2)
;tempy[0:Nx/2-1,Nx/2:*,*] = REVERSE(tempy[Nx/2:*,Nx/2:*,*],1)
;tempy[0:Nx/2-1,0:Nx/2-1,*] = REVERSE(tempy[Nx/2:*,0:Nx/2-1,*],1)

writefits,directory+'B_x.fits',temp
writefits,directory+'B_y.fits',tempy

print, 'Reprojection done'
ENDIF
read,pause
END
