PRO POLYISO2, N, ri, ro, matchpoint=matchpoint, $
	     trunc=trunc, m, filename

; IDL code to compute polytrope + isothermal backgrounds
; with grid point distribution that obeys the criterion of
; constant acoustic travel time between adjacent radial
; points.

; Copyright Shravan Hanasoge 2008
; Email: shravan@solar.stanford.edu
; W. W. Hansen Experimental Physics Laboratory
; Stanford University, Stanford, CA 94305, USA
;

; N - number of radial grid points (usually 300)
; ri - inner radius (ri = 0.95 typically) - units of fractional solar radius
; ro - outer radius (ro = 1.003 works)
; matchpoint - matching point of the polytrope 
;         and isothermal atmospheres - in km
;	  Note that matchpoint <0 and must be at least
;         as large in magnitude as z0.
; trunc - the truncation height of the polytrope
;         the formula now is p = p0 ((Rsol + trunc - r)/z0)^(m+1)
;         or alternately, p0 ((trunc-z)/z0)^(m+1)
;         default is trunc = 0
; m - the polytropic index (I've used 1.5 before)
; filename - the filename, example - 'polyt_model'

; Note that the formalism for the truncated polytrope is taken
; Bogdan, Hindman, Cally, & Charbonneau 1996, ApJ, 465, 406
; 
; z0 - The z0 term below is a parameter than can be changed - it
; represents the truncation level of the polytrope (if trunc=0).
; 

; Note that there aren't too many free parameters. Given m,
; only three of c0, rho0, p0, g, and z0 are arbitrary.

;
; The output file contains 6 columns:
; r/R_solar, sound speed (cm/s), density (g/cc), pressure (dynes/cm^2), gravity (cm/s^2), Gamma_1 (non-dimensional)
;
; Suggested values for ri, ro are:
; ri = 0.95
; ro = 1.003
;
; NOTE: This code requires one file (in the same directory):
; solar_S

;
; To obtain a model with properties rather close to model S, try:
; polyiso2, 300, 0.95, 1.003, z0=200,matchpoint = -50, trunc=300.,1.5,'polyt_model'


Rsol=6.959894677E+10
p0 = 1.21*10.^5.
rho0 = 2.78*10.^(-7.)
;rho0 = 3.5 * 10.^(-7.) ; g/cc - this is the density at the photosphere
;c0 = 7.5*10.^5. ; cm/s - this is the value at the photosphere
;p0 =rho0*c0^2.*m/(m+1.0)
c0 = ((m+1.)/m * p0/rho0)^0.5
ri = ri * Rsol
ro = ro * Rsol

if (keyword_set(matchpoint) eq 0) THEN matchpoint = 0;-z0 * 10.^(-5.)
match = matchpoint * 10.^(5.)
z0 = abs(match)/(m+1.)

IF (keyword_set(trunc) eq 0) THEN trunc = 0.
trunc = trunc * 10.^(5.)

;c0 = ((m+1.0)/m * p0/rho0)^0.5

cpoly = c0 * ((trunc - match)/z0)^(-0.5)
H = (trunc - match)/(m+1.)

IF (match LE 0) THEN BEGIN
 rhopoly = rho0 * exp(-match/H) * ((trunc - match)/z0)^(-m) 
 ppoly = p0 * exp(-match/H) * ((trunc-match)/z0)^(-(m+1)) 

 g = ((m+1.0)*ppoly/(rhopoly*z0))

 rhoiso = rho0
 piso = p0
ENDIF

IF (match GT 0) THEN BEGIN 
 rhopoly = rho0 * (trunc/z0)^(-m) 
 ppoly = p0 * (trunc/z0)^(-(m+1)) 
 
 g = ((m+1.0)*ppoly/(rhopoly*z0))
; H = ppoly*(trunc-match)/(rhopoly*z0*g)
; H = (trunc-match)/(m+1.)

 rhoiso = rhopoly * ((trunc - match)/z0)^(m) * exp(match/H)
 piso = ppoly * ((trunc - match)/z0)^(m + 1) * exp(match/H)
ENDIF

; m is given, number of points N given.
; First generate a 'solar_model'

; To determine the radial grid distribution, first
; estimate the travel times in the polytrope and the
; isothermal layers.

tpoly = 2*z0/cpoly * ( ((Rsol + trunc -ri)/z0)^0.5 - ((trunc - match)/z0)^0.5 ) 
tiso = (ro -Rsol - match)/c0

Npoly = floor(tpoly/(tiso + tpoly) * N + 0.2)
Niso = N - Npoly 

dtaupoly = tpoly/(Npoly-1.)

rad = dblarr(N)
rad[0] = ri

for i=0,Npoly-2 do begin
 rad[i+1] = Rsol + trunc - (((Rsol + trunc -rad[i])/z0)^0.5 - dtaupoly * cpoly/(2.*z0))^2. * z0
endfor
rad[Npoly-1:N-1] = dindgen(Niso + 1)/(Niso*1.0) * (ro -rad[Npoly-1]) + rad[Npoly-1]


pressure = dblarr(N)
density = pressure
gravity = pressure
gam = pressure
sound_speed = pressure

density[0:Npoly-1] = rhopoly * ((Rsol + trunc - rad[0:Npoly-1])/z0)^(m)
density[Npoly:N-1] = rhoiso * exp((Rsol -rad[Npoly:(N-1)])/H) 

pressure[0:Npoly-1] = ppoly * ((Rsol + trunc - rad[0:Npoly-1])/z0)^(m+1)
pressure[Npoly:N-1] = piso * exp((Rsol-rad[Npoly:(N-1)])/H)

sound_speed[0:Npoly-1] = cpoly * ((Rsol + trunc -rad[0:Npoly-1])/z0)^(0.5)
sound_speed[Npoly:(N-1)] = sound_speed[Npoly-1]

gam[*] = (m+1.)/m 

gravity[*] = g



OPENW,1,filename
FOR i=0,N-1 DO BEGIN
PRINTF,1,rad[i]/(Rsol),sound_speed[i],density[i],pressure[i],gravity[i],gam[i],format='(6E21.14)'
ENDFOR
CLOSE,1
read,pause

print, 'The calculation was successful! The output has been written to the file "',filename,'".'
print, 'Now plotting various properties of the background model.'

print,'Gravity (cm/s^2): ',gravity[3]
print, 'Gamma_1: ', gam[50]

h = -1./deriv(rad,alog(density))
dhdr = deriv(rad,h)
ac = 1000.*sound_speed/(4.*!dpi*h)*(1.-2.*dhdr)^0.5

data = dblarr(25,2481)
openr,1,'solar_S'
readf,1,data
close,1
data = reverse(data,2)
rr = data[0,*]/Rsol
pr = data[3,*]
rh = data[4,*]
csol = (data[9,*] * data[3,*]/data[4,*])^0.5
hsol = -1./deriv(data[0,*],alog(rh))
dhdrsol = deriv(data[0,*],hsol)
acsol = 1000.*csol/(4.*!dpi*hsol)*(1.-2.*dhdrsol)^0.5
hp = deriv(rad,alog(pressure))/gam
hpsol = deriv(data[0,*],alog(data[3,*]))/data[9,*]

G=6.6732d-8
Msol=1.989000000E+33
mass = EXP(data[1,*])*Msol
gravsol=G*mass/(data[0,*])^2.0d0

n2 = gravity * (hp + 1./h)
n2sol = gravsol * (hpsol + 1./hsol)

!p.multi=[0,2,3]
!p.charsize=1.6
!p.charthick=1.6
;wset,0
erase

;read,pause
grid = DBLARR(N-1)
FOR I=0,N-2 DO BEGIN
  grid[I] = rad[I+1] - rad[I]
ENDFOR
plot,rad[0:N-2]/Rsol,grid*10.^(-5.),tit='Grid spacing (km)'
oplot,[(1. + match/Rsol), (1. + match/Rsol)],[0,5000.],linesty=3


plot,(rad[180:290]-Rsol)*10.^(-8.), (pressure[180:290]), /ylog,tit='Near-surface pressure',xst=1
oplot,(rr[2000:2450] -1.)*Rsol*10.^(-8.), (pr[2000:2450]),col=180,thick=3
oplot,[match, match]*10.^(-8.),[10.^(-14.),10.^(14.)]

plot,(rad[180:290] - Rsol)*10.^(-8.), (density[180:290]), /ylog,tit='Near-surface density',xst=1
oplot,(rr[2000:2450] -1.)*Rsol*10.^(-8.), (rh[2000:2450]),col=180,thick=3
oplot,[match, match]*10.^(-8.),[10.^(-18.),5000.]

plot,(rad[180:290] - Rsol)*10.^(-8.), (ac[180:290]),tit='Acoustic cutoff',xst=1
oplot,(rr[2000:2450] -1.)*Rsol*10.^(-8.), (acsol[2000:2450]),col=180,thick=3
oplot,[[rad[180], rad[290]]-Rsol]*10.^(-8.),[(ac[290]), (ac[290])],linesty=2
oplot,[match, match]*10.^(-8.),[0.,28.]

plot,(rad[180:290] - Rsol)*10.^(-8.), (sound_speed[180:290]), /ylog,tit='Near-surface sound speed',xst=1
oplot,(rr[2000:2450] -1.)*Rsol*10.^(-8.), (csol[2000:2450]),col=180,thick=3
oplot,[match, match]*10.^(-8.),[1., 10.^(10.)]

plot,(rad[180:290] - Rsol)*10.^(-8.), (n2[180:290]),yrange=[-0.001,0.001],tit='Near-surface B-V freq',xst=1
oplot,(rr[2000:2450] -1.)*Rsol*10.^(-8.), (n2sol[2000:2450]),col=180,thick=3
oplot,[match, match]*10.^(-8.),[-10.^(-2.), 10.^(2.)],linesty=3

!p.multi=0
read,pause
END

